import 'package:flutter/material.dart';
import 'package:slick/configuration/slick_configuration.dart';
import 'package:slick/controls/form_input/slick_text_input.dart';
import 'package:intl/intl.dart';

enum SlickDateMode { Date, TimeOfDay, DateAndTime }

class SlickDateInput extends StatefulWidget {
  SlickDateInput({
    Key key,
    this.initialDateValue,
    this.labelText,
    this.onDateChanged,
    this.onTimeOfDayChanged,
    this.mode = SlickDateMode.Date,
    this.readonly = false,
    this.initialTimeOfDayValue,
  }) : super(key: key);

  final DateTime initialDateValue;
  final TimeOfDay initialTimeOfDayValue;
  final String labelText;
  final ValueChanged<DateTime> onDateChanged;
  final ValueChanged<TimeOfDay> onTimeOfDayChanged;
  final bool readonly;
  final SlickDateMode mode;

  @override
  _SlickDateInputState createState() => _SlickDateInputState();
}

class _SlickDateInputState extends State<SlickDateInput> {
  final TextEditingController _controller = TextEditingController();
  DateTime _currentDate;

  @override
  void initState() {
    if (widget.initialDateValue != null) {
      _currentDate = widget.initialDateValue;
    }

    _currentDate ??= DateTime.now();

    if (widget.initialTimeOfDayValue != null) {
      _currentDate = DateTime(
        _currentDate.year,
        _currentDate.month,
        _currentDate.day,
        widget.initialTimeOfDayValue.hour,
        widget.initialTimeOfDayValue.minute,
        _currentDate.second,
        _currentDate.millisecond,
        _currentDate.microsecond,
      );
    }
    _controller.text = getValue(_currentDate);

    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlickTextInput(
      readOnly: widget.readonly,
      labelText: widget.labelText,
      controller: _controller,
      suffix: Row(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          if (widget.mode == SlickDateMode.Date ||
              widget.mode == SlickDateMode.DateAndTime)
            Builder(
              builder: (context) => IconButton(
                padding: EdgeInsets.all(0.0),
                icon: Icon(
                  Icons.date_range_rounded,
                  color: SlickConfiguration.colors.inputControlBorder,
                  size: 21,
                ),
                onPressed: () async {
                  final picked = await showDatePicker(
                      context: context,
                      initialDate: DateTime.now(),
                      initialDatePickerMode: DatePickerMode.day,
                      firstDate: DateTime(1900),
                      lastDate: DateTime(2030));
                  if (picked != null) {
                    _currentDate = DateTime(
                      picked.year,
                      picked.month,
                      picked.day,
                      _currentDate.hour,
                      _currentDate.minute,
                      _currentDate.second,
                      _currentDate.microsecond,
                      _currentDate.microsecond,
                    );
                    setState(() {
                      _controller.text = getValue(_currentDate);
                    });
                  }
                },
              ),
            ),
          if (widget.mode == SlickDateMode.DateAndTime ||
              widget.mode == SlickDateMode.TimeOfDay)
            Builder(
              builder: (context) => IconButton(
                padding: EdgeInsets.all(0.0),
                icon: Icon(
                  Icons.access_time_rounded,
                  color: SlickConfiguration.colors.inputControlBorder,
                  size: 21,
                ),
                onPressed: () async {
                  final picked = await showTimePicker(
                    context: context,
                    initialTime: TimeOfDay(
                      hour: DateTime.now().hour,
                      minute: DateTime.now().minute,
                    ),
                  );
                  if (picked != null) {
                    _currentDate = DateTime(
                      _currentDate.year,
                      _currentDate.month,
                      _currentDate.day,
                      picked.hour,
                      picked.minute,
                      _currentDate.second,
                      _currentDate.millisecond,
                      _currentDate.microsecond,
                    );

                    setState(() {
                      _controller.text = getValue(_currentDate);
                    });
                  }
                },
              ),
            ),
        ],
      ),
    );
  }

  DateTime fireOnChanged(String val) {
    if (val.isEmpty) widget.onDateChanged(null);
    var dateVal = DateTime.tryParse(val);
    dateVal ??= widget.initialDateValue;
    widget.onDateChanged(dateVal);
    widget.onTimeOfDayChanged(
      TimeOfDay(hour: dateVal.hour, minute: dateVal.minute),
    );
    return dateVal;
  }

  String getValue(DateTime dateValue) {
    if (dateValue == null) return '';

    switch (widget.mode) {
      case SlickDateMode.Date:
        return DateFormat('yyyy-MM-dd').format(dateValue);
      case SlickDateMode.TimeOfDay:
        return DateFormat('HH:mm').format(dateValue);
      case SlickDateMode.DateAndTime:
        return DateFormat('yyyy-MM-dd HH:mm').format(dateValue);
      default:
        return '';
    }
  }
}
